//Copyright 2018 rat de combat sur forum.hardware.fr

/*
This file is part of the firmware for the AD9833-poor men's signal generator.

The firmware is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

The firmware is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with the firmware.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <avr/io.h>
#include <stdint.h>
#include <util/delay.h>
#include "ad9833.h"

#define DB0 0
#define DB1 1
#define DB2 2
#define DB3 3
#define DB4 4
#define DB5 5
#define DB6 6
#define DB7 7
#define DB8 8
#define DB9 9
#define DB10 10
#define DB11 11
#define DB12 12
#define DB13 13
#define DB14 14
#define DB15 15

#define MODE DB1
#define DIV2 DB3
#define OPBITEN DB5
#define SLEEP12 DB6
#define SLEEP1 DB7
#define RESET DB8
#define PSELECT DB10
#define FSELECT DB11
#define HLB DB12
#define B28 D13

#define FREQ0_REG(a) ((1<<DB14)|(a))
#define FREQ1_REG(a) ((1<<DB15)|(a))
#define PHASE0_REG(a) ((1<<DB14)|(1<<DB15)|(a))
#define PHASE1_REG(a) ((1<<DB13)|(1<<DB14)|(1<<DB15)|(a))

static uint16_t config_register=0;

void spi_write16(const uint16_t v)
{
	PORTB&=~(1<<FSYNC);
	SPDR=(v>>8)&0xff;
	while(!(SPSR&(1<<SPIF)));
	SPDR=v&0xff;
	while(!(SPSR&(1<<SPIF)));
	PORTB|=(1<<FSYNC);
}

void reset_ad9833(void)
{
	spi_write16(1<<RESET);
	_delay_ms(5);
	spi_write16(0);
	config_register=0;
}

void write_freq(const uint32_t f, const reg_t reg)
{
	uint32_t val=(uint64_t)f*4194304/390625; //==2**28/25E6
	uint16_t msb=(val>>14)&0x3FFF;
	uint16_t lsb=val&0x3FFF;
	
	if(reg==REG0)
	{
		spi_write16(1<<HLB); //msb
		spi_write16(FREQ0_REG(msb));
		spi_write16(config_register); //lsb
		spi_write16(FREQ0_REG(lsb));
	}
	else
	{
		spi_write16(1<<HLB); //msb
		spi_write16(FREQ1_REG(msb));
		spi_write16(config_register); //lsb
		spi_write16(FREQ1_REG(lsb));
	}
}

void write_phase(const uint16_t p, const reg_t reg) //phase in degrees
{
	uint16_t val=((uint32_t)p*4096)/360;
	
	if(reg==REG0)
	{
		spi_write16(PHASE0_REG(val));
	}
	else
	{
		spi_write16(PHASE1_REG(val));
	}
}

void write_waveform(const waveform_t w)
{
	switch(w)
	{
		case SINE: //~600mVpp
			config_register&=~((1<<OPBITEN)|(1<<MODE));
			break;
		case TRIANGLE: //~600mVpp
			config_register&=~(1<<OPBITEN);
			config_register|=(1<<MODE);
			break;
		case SQUARE: //5Vpp
			config_register&=~(1<<MODE);
			config_register|=(1<<OPBITEN)|(1<<DIV2);
			break;
		case SQUARE_2: //5Vpp
			config_register&=~((1<<MODE)|(1<<DIV2));
			config_register|=(1<<OPBITEN);
			break;
	}
	spi_write16(config_register);
}

void select_register(const reg_type_t t, const reg_t reg)
{
	if(t==FREQ)
	{
		if(reg==REG0)
			config_register&=~(1<<FSELECT);
		else
			config_register|=(1<<FSELECT);
	}
	else //PHASE
	{
		if(reg==REG0)
			config_register&=~(1<<PSELECT);
		else
			config_register|=(1<<PSELECT);
	}
	spi_write16(config_register);
}
