//Copyright 2018 rat de combat sur forum.hardware.fr

/*
This file is part of the firmware for the AD9833-poor men's signal generator.

The firmware is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

The firmware is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with the firmware.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <avr/io.h>
#include <util/delay.h>
#include <stdint.h>
#define BAUD 9600
#include <util/setbaud.h>
#include <stdio.h>
#include <avr/interrupt.h>

#include "parser.h"
#include "ad9833.h"

//avrdude -p m8 -c usbasp-clone -U flash:w:avr.elf

#define LOW(a) (a&0xFF)
#define HIGH(a) ((a>>8)&0xFF)

#define BUFFER_MAX 10

volatile uint8_t command_received=0;
volatile uint8_t index_serial=0;
volatile char serial_buffer[BUFFER_MAX+1];

void uart_putchar(char c, FILE *stream);

ISR(USART_RXC_vect)
{
	char c=UDR;
	if(c=='\r')
	{
		serial_buffer[index_serial]='\0';
		command_received=1;
	}
	else
	{
		if(index_serial<BUFFER_MAX)
			serial_buffer[index_serial++]=c;
	}
}

int main(void)
{
	DDRB|=(1<<FSYNC)|(1<<SS)|(1<<MOSI)|(1<<SCK); //SS output!!
	PORTB|=(1<<FSYNC);
	
	//init uart
	UBRRL=LOW(UBRR_VALUE);
	UBRRH=HIGH(UBRR_VALUE);
	UCSRB|=(1<<TXEN)|(1<<RXEN)|(1<<RXCIE);
	
	FILE uart_output = FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);
	stdout = &uart_output;
	
	//init spi F_CPU/4
	//spi enable, master mode, mode 2
	SPCR=(1<<SPE)|(1<<MSTR)|(1<<CPOL);
	
	reset_all();
	
	printf("ready...\r\n");
	
	sei();
	
	while(1)
	{
		if(command_received)
		{
			parse_command((char *)serial_buffer); //no longer volatile
			
			index_serial=0;
			command_received=0;
		}
	}
	
	return 0;
}

void uart_putchar(char c, FILE *stream)
{
	while(!(UCSRA&(1<<UDRE)));
	UDR=c;
}
